@@ -1,6 +1,4 @@
-"""
-A pytest plugin useful to test tox itself (and its plugins).
-"""
+"""A pytest plugin useful to test tox itself (and its plugins)."""
 from __future__ import annotations
 
 import inspect
@@ -14,45 +12,36 @@ import warnings
 from contextlib import closing, contextmanager
 from pathlib import Path
 from types import ModuleType, TracebackType
-from typing import TYPE_CHECKING, Any, Callable, Iterator, Sequence, cast
-from unittest.mock import MagicMock
+from typing import TYPE_CHECKING, Any, Callable, Iterator, Protocol, Sequence, cast
 
 import pytest
-from _pytest.capture import CaptureFixture as _CaptureFixture
-from _pytest.config import Config as PyTestConfig
-from _pytest.config.argparsing import Parser
 from _pytest.fixtures import SubRequest
-from _pytest.logging import LogCaptureFixture
-from _pytest.monkeypatch import MonkeyPatch
-from _pytest.python import Function
-from _pytest.tmpdir import TempPathFactory
 from devpi_process import IndexServer
-from pytest_mock import MockerFixture
 from virtualenv.info import fs_supports_symlink
 
 import tox.run
-from tox.config.sets import EnvConfigSet
 from tox.execute.api import Execute, ExecuteInstance, ExecuteOptions, ExecuteStatus, Outcome
 from tox.execute.request import ExecuteRequest, shell_cmd
-from tox.execute.stream import SyncWrite
 from tox.plugin import manager
 from tox.report import LOGGER, OutErr
 from tox.run import run as tox_run
 from tox.run import setup_state as previous_setup_state
 from tox.session.cmd.run.parallel import ENV_VAR_KEY
-from tox.session.state import State
 from tox.tox_env import api as tox_env_api
 from tox.tox_env.api import ToxEnv
 
-if sys.version_info >= (3, 8):  # pragma: no cover (py38+)
-    from typing import Protocol
-else:  # pragma: no cover (<py38)
-    from typing_extensions import Protocol
-
 if TYPE_CHECKING:
-    CaptureFixture = _CaptureFixture[str]
+    from unittest.mock import MagicMock
+
+    from pytest_mock import MockerFixture
+
+    from tox.config.sets import EnvConfigSet
+    from tox.execute.stream import SyncWrite
+    from tox.session.state import State
+
+    CaptureFixture = pytest.CaptureFixture[str]
 else:
-    CaptureFixture = _CaptureFixture
+    CaptureFixture = pytest.CaptureFixture
 
 os.environ["PIP_DISABLE_PIP_VERSION_CHECK"] = "1"
 os.environ["PIP_NO_PYTHON_VERSION_WARNING"] = "1"
@@ -71,9 +60,9 @@ def ensure_logging_framework_not_altered() -> Iterator[None]:  # noqa: PT004
 
 @pytest.fixture(autouse=True)
 def _disable_root_tox_py(request: SubRequest, mocker: MockerFixture) -> Iterator[None]:
-    """unless this is a plugin test do not allow loading toxfile.py"""
+    """Unless this is a plugin test do not allow loading toxfile.py."""
     if request.node.get_closest_marker("plugin_test"):  # unregister inline plugin
-        module, load_inline = None, manager._load_inline
+        module, load_inline = None, manager._load_inline  # noqa: SLF001
 
         def _load_inline(path: Path) -> ModuleType | None:  # register only on first run, and unregister at end
             nonlocal module
@@ -92,7 +81,7 @@ def _disable_root_tox_py(request: SubRequest, mocker: MockerFixture) -> Iterator
 @contextmanager
 def check_os_environ() -> Iterator[None]:
     old = os.environ.copy()
-    to_clean = {k: os.environ.pop(k, None) for k in {ENV_VAR_KEY, "TOX_WORK_DIR", "PYTHONPATH", "COV_CORE_CONTEXT"}}
+    to_clean = {k: os.environ.pop(k, None) for k in (ENV_VAR_KEY, "TOX_WORK_DIR", "PYTHONPATH", "COV_CORE_CONTEXT")}
 
     yield
 
@@ -119,29 +108,29 @@ def check_os_environ() -> Iterator[None]:
 
 
 @pytest.fixture(autouse=True)
-def check_os_environ_stable(monkeypatch: MonkeyPatch) -> Iterator[None]:  # noqa: PT004
+def check_os_environ_stable(monkeypatch: pytest.MonkeyPatch) -> Iterator[None]:  # noqa: PT004
     with check_os_environ():
         yield
         monkeypatch.undo()
 
 
 @pytest.fixture(autouse=True)
-def no_color(monkeypatch: MonkeyPatch, check_os_environ_stable: None) -> None:  # noqa: PT004, U100
+def no_color(monkeypatch: pytest.MonkeyPatch, check_os_environ_stable: None) -> None:  # noqa: ARG001, PT004
     monkeypatch.setenv("NO_COLOR", "yes")
 
 
 class ToxProject:
-    def __init__(
+    def __init__(  # noqa: PLR0913
         self,
         files: dict[str, Any],
         base: Path | None,
         path: Path,
         capfd: CaptureFixture,
-        monkeypatch: MonkeyPatch,
+        monkeypatch: pytest.MonkeyPatch,
         mocker: MockerFixture,
     ) -> None:
         self.path: Path = path
-        self.monkeypatch: MonkeyPatch = monkeypatch
+        self.monkeypatch: pytest.MonkeyPatch = monkeypatch
         self.mocker = mocker
         self._capfd = capfd
         self._setup_files(self.path, base, files)
@@ -153,10 +142,12 @@ class ToxProject:
         dest.mkdir(exist_ok=True)
         for key, value in content.items():
             if not isinstance(key, str):
-                raise TypeError(f"{key!r} at {dest}")  # pragma: no cover
+                msg = f"{key!r} at {dest}"
+                raise TypeError(msg)  # pragma: no cover
             at_path = dest / key
             if callable(value):
-                value = textwrap.dedent("\n".join(inspect.getsourcelines(value)[0][1:]))
+                value = textwrap.dedent("\n".join(inspect.getsourcelines(value)[0][1:]))  # noqa: PLW2901
+                value = f"from __future__ import annotations\n{value}"  # noqa: PLW2901
             if isinstance(value, dict):
                 at_path.mkdir(exist_ok=True)
                 ToxProject._setup_files(at_path, None, value)
@@ -168,9 +159,9 @@ class ToxProject:
                 msg = f"could not handle {at_path / key} with content {value!r}"  # pragma: no cover
                 raise TypeError(msg)  # pragma: no cover
 
-    def patch_execute(self, handle: Callable[[ExecuteRequest], int | None]) -> MagicMock:
+    def patch_execute(self, handle: Callable[[ExecuteRequest], int | None]) -> MagicMock:  # noqa: C901
         class MockExecute(Execute):
-            def __init__(self, colored: bool, exit_code: int) -> None:
+            def __init__(self, colored: bool, exit_code: int) -> None:  # noqa: FBT001
                 self.exit_code = exit_code
                 super().__init__(colored)
 
@@ -192,17 +183,17 @@ class ToxProject:
             def exit_code(self) -> int | None:
                 return self._exit_code
 
-            def wait(self, timeout: float | None = None) -> int | None:  # noqa: U100
+            def wait(self, timeout: float | None = None) -> int | None:  # noqa: ARG002
                 return self._exit_code
 
-            def write_stdin(self, content: str) -> None:  # noqa: U100
+            def write_stdin(self, content: str) -> None:  # noqa: ARG002
                 return None  # pragma: no cover
 
             def interrupt(self) -> None:
                 return None  # pragma: no cover
 
         class MockExecuteInstance(ExecuteInstance):
-            def __init__(
+            def __init__(  # noqa: PLR0913
                 self,
                 request: ExecuteRequest,
                 options: ExecuteOptions,
@@ -218,9 +209,9 @@ class ToxProject:
 
             def __exit__(
                 self,
-                exc_type: type[BaseException] | None,  # noqa: U100
-                exc_val: BaseException | None,  # noqa: U100
-                exc_tb: TracebackType | None,  # noqa: U100
+                exc_type: type[BaseException] | None,
+                exc_val: BaseException | None,
+                exc_tb: TracebackType | None,
             ) -> None:
                 pass
 
@@ -234,17 +225,16 @@ class ToxProject:
             executor: Execute,
             out_err: OutErr,
             request: ExecuteRequest,
-            show: bool,
+            show: bool,  # noqa: FBT001
         ) -> Iterator[ExecuteStatus]:
             exit_code = handle(request)
             if exit_code is not None:
-                executor = MockExecute(colored=executor._colored, exit_code=exit_code)
+                executor = MockExecute(colored=executor._colored, exit_code=exit_code)  # noqa: SLF001
             with original_execute_call(self, executor, out_err, request, show) as status:
                 yield status
 
-        original_execute_call = ToxEnv._execute_call
-        result = self.mocker.patch.object(ToxEnv, "_execute_call", side_effect=_execute_call, autospec=True)
-        return result
+        original_execute_call = ToxEnv._execute_call  # noqa: SLF001
+        return self.mocker.patch.object(ToxEnv, "_execute_call", side_effect=_execute_call, autospec=True)
 
     @property
     def structure(self) -> dict[str, Any]:
@@ -261,8 +251,8 @@ class ToxProject:
 
     @contextmanager
     def chdir(self, to: Path | None = None) -> Iterator[None]:
-        cur_dir = os.getcwd()
-        os.chdir(str(to or self.path))
+        cur_dir = Path.cwd()
+        os.chdir(to or self.path)
         try:
             yield
         finally:
@@ -283,7 +273,7 @@ class ToxProject:
             with self.monkeypatch.context() as m:
                 m.setattr(tox_env_api, "_CWD", self.path)
                 m.setattr(tox.run, "setup_state", our_setup_state)
-                m.setattr(sys, "argv", [sys.executable, "-m", "tox"] + list(args))
+                m.setattr(sys, "argv", [sys.executable, "-m", "tox", *list(args)])
                 m.setenv("VIRTUALENV_SYMLINK_APP_DATA", "1")
                 m.setenv("VIRTUALENV_SYMLINKS", "1")
                 m.setenv("VIRTUALENV_PIP", "embed")
@@ -294,7 +284,8 @@ class ToxProject:
                 except SystemExit as exception:
                     code = exception.code
                 if code is None:  # pragma: no branch
-                    raise RuntimeError("exit code not set")
+                    msg = "exit code not set"
+                    raise RuntimeError(msg)
             out, err = self._capfd.readouterr()
             return ToxRunOutcome(args, self.path, cast(int, code), out, err, state)
 
@@ -317,16 +308,24 @@ def enable_pep517_backend_coverage() -> Iterator[None]:  # noqa: PT004
         result.append("COV_*")
         return result
 
-    previous = Pep517VirtualEnvPackager._default_pass_env
+    previous = Pep517VirtualEnvPackager._default_pass_env  # noqa: SLF001
     try:
-        Pep517VirtualEnvPackager._default_pass_env = default_pass_env  # type: ignore
+        Pep517VirtualEnvPackager._default_pass_env = default_pass_env  # type: ignore[assignment]  # noqa: SLF001
         yield
     finally:
-        Pep517VirtualEnvPackager._default_pass_env = previous  # type: ignore
+        Pep517VirtualEnvPackager._default_pass_env = previous  # type: ignore[method-assign]  # noqa: SLF001
 
 
 class ToxRunOutcome:
-    def __init__(self, cmd: Sequence[str], cwd: Path, code: int, out: str, err: str, state: State | None) -> None:
+    def __init__(  # noqa: PLR0913
+        self,
+        cmd: Sequence[str],
+        cwd: Path,
+        code: int,
+        out: str,
+        err: str,
+        state: State | None,
+    ) -> None:
         extended_cmd = [sys.executable, "-m", "tox"]
         extended_cmd.extend(cmd)
         self.cmd: list[str] = extended_cmd
@@ -339,7 +338,8 @@ class ToxRunOutcome:
     @property
     def state(self) -> State:
         if self._state is None:
-            raise RuntimeError("no state")
+            msg = "no state"
+            raise RuntimeError(msg)
         return self._state
 
     def env_conf(self, name: str) -> EnvConfigSet:
@@ -350,11 +350,11 @@ class ToxRunOutcome:
         return self.code == Outcome.OK
 
     def assert_success(self) -> None:
-        assert self.success, repr(self)
+        assert self.success, repr(self)  # noqa: S101
 
     def assert_failed(self, code: int | None = None) -> None:
         status_match = self.code != 0 if code is None else self.code == code
-        assert status_match, f"should be {code}, got {self}"
+        assert status_match, f"should be {code}, got {self}"  # noqa: S101
 
     def __repr__(self) -> str:
         return "\n".join(
@@ -379,13 +379,13 @@ class ToxRunOutcome:
         if regex:
             self.matches(out, self.out, re.MULTILINE | re.DOTALL)
         else:
-            assert self.out == out
+            assert self.out == out  # noqa: S101
         if dedent:
             err = textwrap.dedent(err).lstrip()
         if regex:
             self.matches(err, self.err, re.MULTILINE | re.DOTALL)
         else:
-            assert self.err == err
+            assert self.err == err  # noqa: S101
 
     @staticmethod
     def matches(pattern: str, text: str, flags: int = 0) -> None:
@@ -395,17 +395,17 @@ class ToxRunOutcome:
             match = re.match(pattern, text, flags)
             if match is None:
                 warnings.warn("install the re-assert PyPI package for bette error message", UserWarning, stacklevel=1)
-            assert match
+            assert match  # noqa: S101
         else:
-            assert Matches(pattern, flags=flags) == text
+            assert Matches(pattern, flags=flags) == text  # noqa: S101
 
 
 class ToxProjectCreator(Protocol):
     def __call__(
         self,
-        files: dict[str, Any],  # noqa: U100
-        base: Path | None = None,  # noqa: U100
-        prj_path: Path | None = None,  # noqa: U100
+        files: dict[str, Any],
+        base: Path | None = None,
+        prj_path: Path | None = None,
     ) -> ToxProject:
         ...
 
@@ -414,18 +414,18 @@ class ToxProjectCreator(Protocol):
 def init_fixture(
     tmp_path: Path,
     capfd: CaptureFixture,
-    monkeypatch: MonkeyPatch,
+    monkeypatch: pytest.MonkeyPatch,
     mocker: MockerFixture,
 ) -> ToxProjectCreator:
     def _init(files: dict[str, Any], base: Path | None = None, prj_path: Path | None = None) -> ToxProject:
-        """create tox  projects"""
+        """Create tox  projects."""
         return ToxProject(files, base, prj_path or tmp_path / "p", capfd, monkeypatch, mocker)
 
     return _init
 
 
 @pytest.fixture()
-def empty_project(tox_project: ToxProjectCreator, monkeypatch: MonkeyPatch) -> ToxProject:
+def empty_project(tox_project: ToxProjectCreator, monkeypatch: pytest.MonkeyPatch) -> ToxProject:
     project = tox_project({"tox.ini": ""})
     monkeypatch.chdir(project.path)
     return project
@@ -434,24 +434,24 @@ def empty_project(tox_project: ToxProjectCreator, monkeypatch: MonkeyPatch) -> T
 _RUN_INTEGRATION_TEST_FLAG = "--run-integration"
 
 
-def pytest_addoption(parser: Parser) -> None:
+def pytest_addoption(parser: pytest.Parser) -> None:
     parser.addoption(_RUN_INTEGRATION_TEST_FLAG, action="store_true", help="run the integration tests")
 
 
-def pytest_configure(config: PyTestConfig) -> None:
+def pytest_configure(config: pytest.Config) -> None:
     config.addinivalue_line("markers", "integration")
     config.addinivalue_line("markers", "plugin_test")
 
 
-@pytest.hookimpl(trylast=True)  # type: ignore # not typed decorator
-def pytest_collection_modifyitems(config: PyTestConfig, items: list[Function]) -> None:
+@pytest.hookimpl(trylast=True)
+def pytest_collection_modifyitems(config: pytest.Config, items: list[pytest.Function]) -> None:
     # do not require flags if called directly
     if len(items) == 1:  # pragma: no cover # hard to test
         return
 
     skip_int = pytest.mark.skip(reason=f"integration tests not run (no {_RUN_INTEGRATION_TEST_FLAG} flag)")
 
-    def is_integration(test_item: Function) -> bool:
+    def is_integration(test_item: pytest.Function) -> bool:
         return test_item.get_closest_marker("integration") is not None
 
     integration_enabled = config.getoption(_RUN_INTEGRATION_TEST_FLAG)
@@ -463,7 +463,7 @@ def pytest_collection_modifyitems(config: PyTestConfig, items: list[Function]) -
     items.sort(key=is_integration)
 
 
-def enable_pypi_server(monkeypatch: MonkeyPatch, url: str | None) -> None:
+def enable_pypi_server(monkeypatch: pytest.MonkeyPatch, url: str | None) -> None:
     if url is None:  # pragma: no cover # only one of the branches can be hit depending on env
         monkeypatch.delenv("PIP_INDEX_URL", raising=False)
     else:  # pragma: no cover
@@ -473,7 +473,7 @@ def enable_pypi_server(monkeypatch: MonkeyPatch, url: str | None) -> None:
 
 
 @pytest.fixture(scope="session")
-def pypi_server(tmp_path_factory: TempPathFactory) -> Iterator[IndexServer]:
+def pypi_server(tmp_path_factory: pytest.TempPathFactory) -> Iterator[IndexServer]:
     # takes around 2.5s
     path = tmp_path_factory.mktemp("pypi")
     with IndexServer(path) as server:
@@ -489,8 +489,8 @@ def _invalid_index_fake_port() -> int:  # noqa: PT005
 
 
 @pytest.fixture(autouse=True)
-def disable_pip_pypi_access(_invalid_index_fake_port: int, monkeypatch: MonkeyPatch) -> tuple[str, str | None]:
-    """set a fake pip index url, tests that want to use a pypi server should create and overwrite this"""
+def disable_pip_pypi_access(_invalid_index_fake_port: int, monkeypatch: pytest.MonkeyPatch) -> tuple[str, str | None]:
+    """Set a fake pip index url, tests that want to use a pypi server should create and overwrite this."""
     previous_url = os.environ.get("PIP_INDEX_URL")
     new_url = f"http://localhost:{_invalid_index_fake_port}/bad-pypi-server"
     monkeypatch.setenv("PIP_INDEX_URL", new_url)
@@ -502,9 +502,9 @@ def disable_pip_pypi_access(_invalid_index_fake_port: int, monkeypatch: MonkeyPa
 @pytest.fixture(name="enable_pip_pypi_access")
 def enable_pip_pypi_access_fixture(
     disable_pip_pypi_access: tuple[str, str | None],
-    monkeypatch: MonkeyPatch,
+    monkeypatch: pytest.MonkeyPatch,
 ) -> str | None:
-    """set a fake pip index url, tests that want to use a pypi server should create and overwrite this"""
+    """Set a fake pip index url, tests that want to use a pypi server should create and overwrite this."""
     _, previous_url = disable_pip_pypi_access
     enable_pypi_server(monkeypatch, previous_url)
     return previous_url
@@ -513,13 +513,17 @@ def enable_pip_pypi_access_fixture(
 def register_inline_plugin(mocker: MockerFixture, *args: Callable[..., Any]) -> None:
     frame_info = inspect.stack()[1]
     caller_module = inspect.getmodule(frame_info[0])
-    assert caller_module is not None
+    assert caller_module is not None  # noqa: S101
     plugin = ModuleType(f"{caller_module.__name__}|{frame_info[3]}")
     plugin.__file__ = caller_module.__file__
     plugin.__dict__.update({f.__name__: f for f in args})
     mocker.patch("tox.plugin.manager.load_inline", return_value=plugin)
 
 
+LogCaptureFixture = pytest.LogCaptureFixture
+TempPathFactory = pytest.TempPathFactory
+MonkeyPatch = pytest.MonkeyPatch
+
 __all__ = (
     "CaptureFixture",
     "LogCaptureFixture",
